@@ -9,7 +9,8 @@
 
 """Console script for raytraverse."""
 import os
-import sys
+import shutil
+from glob import glob
 
 import numpy as np
 
@@ -17,46 +18,17 @@ from clasp import click
 import clasp.click_ext as clk
 
 import raytraverse
-from raytraverse.lightfield import LightPlaneKD, DayLightPlaneKD
+from raytraverse import translate
+from raytraverse.lightfield import LightPlaneKD, DayLightPlaneKD, ResultAxis
+from raytraverse.lightfield import LightResult
 from raytraverse.mapper import PlanMapper, SkyMapper, ViewMapper
+from raytraverse.utility.cli import np_load, np_load_safe, shared_pull, pull_decs
 from raytraverse.scene import Scene
 from raytraverse.renderer import Rtrace, Rcontrib
 from raytraverse.sampler import SkySamplerPt, SamplerArea, SamplerSuns
 from raytraverse.sky import SkyData, skycalc
 
 
-@clk.pretty_name("NPY, TSV, FLOATS,FLOATS")
-def np_load(ctx, param, s):
-    """read np array from command line
-
-    trys np.load (numpy binary), then np.loadtxt (space seperated txt file)
-    then split row by spaces and columns by commas.
-    """
-    if s is None:
-        return s
-    if os.path.exists(s):
-        try:
-            ar = np.load(s)
-        except ValueError:
-            ar = np.loadtxt(s)
-        if len(ar.shape) == 1:
-            ar = ar.reshape(1, -1)
-        return ar
-    else:
-        return np.array([[float(i) for i in j.split(',')] for j in s.split()])
-
-
-@clk.pretty_name("NPY, TSV, FLOATS,FLOATS, FILE")
-def np_load_safe(ctx, param, s):
-    try:
-        return np_load(ctx, param, s)
-    except ValueError as ex:
-        if os.path.exists(s):
-            return s
-        else:
-            raise ex
-
-
 @click.group(chain=True, invoke_without_command=True)
 @click.option('-out', type=click.Path(file_okay=False))
 @click.option('-config', '-c', type=click.Path(exists=True),
@@ -227,9 +199,15 @@ candidates within adaptive grid.""")
               help="if True, use location from epw/wea argument to -loc as a"
                    " transit mask (like -loc option 1.) instead of as a list"
                    " of candidate sun positions.")
+@click.option("--printdata/--no-printdata", default=False,
+              help="if True, print skymapper sun positions (either boundary or "
+                   "candidates in xyz coordinates)")
+@click.option("-printlevel", type=int,
+              help="print a set of sun positions at sampling level "
+                   "(overrides printdata)")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def suns(ctx, loc=None, opts=False, debug=False, version=False, epwloc=False,
-         **kwargs):
+         printdata=False, printlevel=None, **kwargs):
     """define solar sampling space
 
     Effects
@@ -246,6 +224,21 @@ def suns(ctx, loc=None, opts=False, debug=False, version=False, epwloc=False,
         except ValueError:
             pass
     ctx.obj['skymapper'] = SkyMapper(loc=loc, **kwargs)
+    if printdata or printlevel is not None:
+        sm = ctx.obj['skymapper']
+        if printlevel is not None:
+            xyz = sm.solar_grid(level=printlevel)
+        elif sm.solarbounds is None:
+            xyz = sm.uv2xyz(sm.candidates)
+        else:
+            xyz = sm.uv2xyz(sm.solarbounds.vertices)
+            xyz = xyz[np.logical_not(np.isnan(xyz[:, 0]))]
+        click.echo("columns: 0:x 1:y 2:z 3:x(anglular) 4:y(angular) 5:u 6:v", err=True)
+        uv = sm.xyz2uv(xyz)
+        axy = translate.xyz2xy(xyz)
+        for x, u, a in zip(xyz, uv, axy):
+            print("{: >10.05f} {: >10.05f} {: >10.05f} {: >10.05f} {: >10.05f} "
+                  "{: >10.05f} {: >10.05f}".format(*x, *a, *u))
 
 
 @main.command()
@@ -311,10 +304,13 @@ def skydata(ctx, wea=None, name="skydata", loc=None, reload=True, opts=False,
               help='the final directional sampling resolution, yielding a'
                    ' grid of potential samples at 2^fdres x 2^fdres per'
                    ' hemisphere')
+@click.option('-dcompargs', default='-ab 1',
+              help="additional arguments for running direct component. when "
+                   "using, set -ab in sunengine.rayargs to this ab minus one.")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def skyengine(ctx, accuracy=1.0, idres=5, rayargs=None, default_args=True,
-              skyres=10.0, fdres=9, opts=False, debug=False, version=None,
-              **kwargs):
+              skyres=10.0, fdres=9, dcompargs='-ab 1', usedecomp=False,
+              opts=False, debug=False, version=None, **kwargs):
     """initialize engine for skyrun
 
     Effects
@@ -325,6 +321,11 @@ def skyengine(ctx, accuracy=1.0, idres=5, rayargs=None, default_args=True,
     if 'scene' not in ctx.obj:
         clk.invoke_dependency(ctx, scene, reload=True, overwrite=False)
     scn = ctx.obj['scene']
+    if usedecomp:
+        if rayargs is not None:
+            rayargs += " " + dcompargs
+        else:
+            rayargs = dcompargs
     rcontrib = Rcontrib(rayargs=rayargs, scene=scn.scene, skyres=skyres,
                         default_args=default_args)
     ctx.obj['skyengine'] = SkySamplerPt(scn, rcontrib, accuracy=accuracy,
@@ -356,10 +357,15 @@ def skyengine(ctx, accuracy=1.0, idres=5, rayargs=None, default_args=True,
                    "the specular guide will either over sample (including "
                    "direct views) or under sample (miss specular reflections) "
                    "depending on this setting.")
+@click.option('-dcompargs', default='-ab 0',
+              help="additional arguments for running direct component. when "
+                   "using, set -ab in skyengine.rayargs to this ab plus one.")
+@click.option("--usedcomp/--no-usedcomp", default=False,
+              help="configure Rtrace as a dcomp run (appends dcompargs)")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def sunengine(ctx, accuracy=1.0, idres=5, rayargs=None, default_args=True,
-              fdres=10, slimit=0.01, maxspec=0.2, opts=False, debug=False,
-              version=None, **kwargs):
+              fdres=10, slimit=0.01, maxspec=0.2, dcompargs='-ab 0',
+              usedcomp=False, opts=False, debug=False, version=None, **kwargs):
     """initialize engine for sunrun
 
     Effects
@@ -369,12 +375,20 @@ def sunengine(ctx, accuracy=1.0, idres=5, rayargs=None, default_args=True,
     if 'scene' not in ctx.obj:
         clk.invoke_dependency(ctx, scene, reload=True, overwrite=False)
     scn = ctx.obj['scene']
-    rtrace = Rtrace(rayargs=rayargs, scene=scn.scene, default_args=default_args)
+    if usedcomp:
+        nproc = 1
+        if rayargs is not None:
+            rayargs += " " + dcompargs
+        else:
+            rayargs = dcompargs
+    else:
+        nproc = None
+    rtrace = Rtrace(rayargs=rayargs, scene=scn.scene, default_args=default_args,
+                    nproc=nproc)
     ptkwargs = dict(slimit=slimit, maxspec=maxspec, accuracy=accuracy,
                     idres=idres, fdres=fdres)
     ctx.obj['sunengine'] = dict(engine=rtrace, ptkwargs=ptkwargs)
 
-
 sample_opts = [
  click.option("-accuracy", default=1.0,
               help="parameter to set threshold at sampling level relative to "
@@ -384,18 +398,23 @@ sample_opts = [
               help="number of levels to sample (final resolution will be "
                    "ptres/2^(nlev-1))"),
  click.option("--jitter/--no-jitter", default=True,
-              help="jitter samples on plane within adaptive sampling grid")
+              help="jitter samples on plane within adaptive sampling grid"),
+ click.option("--plotp/--no-plotp", default=False,
+              help="plot pdfs and sample vecs for each level")
     ]
 
 
 @main.command()
 @clk.shared_decs(sample_opts)
+@click.option("--dcomp/--no-dcomp", default=True,
+              help="calculate indirect sun component from skyrun "
+                   "(2-phase DDS / daysim / 5-phase)")
 @click.option("--overwrite/--no-overwrite", default=False,
               help="If True, reruns sampler when invoked, otherwise will first"
                    " attempt to load results")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
-def skyrun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False, opts=False,
-           debug=False, version=None):
+def skyrun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False, dcomp=True,
+           plotp=False, opts=False, debug=False, version=None):
     """run scene under sky for a set of points (defined by area)
 
     Effects
@@ -428,10 +447,26 @@ def skyrun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False, opts=False,
         skyfield = LightPlaneKD(scn, f"{scn.outdir}/{pm.name}/sky_points"
                                 f".tsv", pm, "sky")
     except OSError:
-        skyfield = skysampler.run(pm)
+        skyfield = skysampler.run(pm, plotp=plotp)
     else:
         click.echo(f"Sky Lightfield reloaded from {scn.outdir}/{pm.name} "
                    f"use --overwrite to rerun", err=True)
+    if dcomp:
+        try:
+            if overwrite:
+                raise OSError
+            dskyfield = LightPlaneKD(scn,
+                                     f"{scn.outdir}/{pm.name}/skydcomp_points"
+                                     f".tsv", pm, "skydcomp")
+        except OSError:
+            skengine.engine.reset()
+            clk.invoke_dependency(ctx, skyengine, usedecomp=True)
+            skysampler = SamplerArea(scn, skengine, accuracy=accuracy,
+                                     nlev=nlev, jitter=jitter)
+            dskyfield = skysampler.repeat(skyfield, "skydcomp")
+        ctx.obj['dskyfield'] = dskyfield
+    else:
+        ctx.obj['dskyfield'] = None
     ctx.obj['skyfield'] = skyfield
 
 
@@ -449,8 +484,11 @@ def skyrun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False, opts=False,
                    "candidate SkyMappers, only affects weighting of selecting"
                    " candidates in the same grid true positions are still "
                    "used")
-@click.option("--recover/--no-recover", default=False,
+@click.option("--recover/--no-recover", default=True,
               help="If True, recovers existing sampling")
+@click.option("--overwrite/--no-overwrite", default=False,
+              help="If True, reruns sampler when invoked, otherwise will first"
+                   " attempt to load results")
 @click.option("--guided/--no-guided", default=True,
               help="If True, uses skysampling results to guide sun sampling "
                    "this is necessary if the model has any specular "
@@ -458,7 +496,8 @@ def skyrun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False, opts=False,
                    " called yet.")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def sunrun(ctx, srcaccuracy=1.0, srcnlev=3, srcjitter=True, recover=False,
-           guided=True, opts=False, debug=False, version=None, **kwargs):
+           overwrite=False, guided=True, plotp=False, opts=False, debug=False,
+           version=None, **kwargs):
     """run scene for a set of suns (defined by suns) for a set of points
     (defined by area)
 
@@ -479,32 +518,58 @@ def sunrun(ctx, srcaccuracy=1.0, srcnlev=3, srcjitter=True, recover=False,
     if 'planmapper' not in ctx.obj:
         clk.invoke_dependency(ctx, area)
     pm = ctx.obj['planmapper']
-    if 'sunengine' not in ctx.obj:
-        clk.invoke_dependency(ctx, sunengine)
-    snengine = ctx.obj['sunengine']['engine']
-    ptkwargs = ctx.obj['sunengine']['ptkwargs']
     if 'skymapper' not in ctx.obj:
         clk.invoke_dependency(ctx, suns)
     skmapper = ctx.obj['skymapper']
-
     if guided and 'skyfield' not in ctx.obj:
         clk.invoke_dependency(ctx, skyrun, overwrite=False)
     if guided:
         specguide = ctx.obj['skyfield']
+        dcomp = ctx.obj['dskyfield']
     else:
         specguide = None
+        dcomp = None
+    if 'sunengine' not in ctx.obj:
+        clk.invoke_dependency(ctx, sunengine, usedcomp=dcomp is not None)
+    snengine = ctx.obj['sunengine']['engine']
+    ptkwargs = ctx.obj['sunengine']['ptkwargs']
     if ctx.obj['static_points']:
         kwargs.update(nlev=1, jitter=False)
     sunsampler = SamplerSuns(scn, snengine, accuracy=srcaccuracy, nlev=srcnlev,
                              jitter=srcjitter, ptkwargs=ptkwargs,
                              areakwargs=kwargs)
-    dfield = sunsampler.run(skmapper, pm, specguide=specguide, recover=recover)
+    sunfile = f"{scn.outdir}/{pm.name}/{skmapper.name}_sunpositions.tsv"
+    if overwrite:
+        shutil.rmtree(sunfile, ignore_errors=True)
+        for src in glob(f"{scn.outdir}/{pm.name}/{skmapper.name}_sun_*"):
+            shutil.rmtree(src, ignore_errors=True)
+        for src in glob(f"{scn.outdir}/{pm.name}/i_{skmapper.name}_sun_*"):
+            shutil.rmtree(src, ignore_errors=True)
+    dfield = sunsampler.run(skmapper, pm, specguide=specguide, recover=recover,
+                            plotp=plotp, pfish=False)
+    if dcomp is not None:
+        dfield = dfield.add_indirect_to_suns(dcomp)
     ctx.obj['daylightfield'] = dfield
 
 
 @main.command()
-@click.option("-sensor", callback=clk.split_float,
-              help="the sensor location (6 component vector 'x y z dx dy dz')")
+@click.option("-sensors", callback=np_load,
+              help="sensor points, this can be a .npy file, a whitespace "
+                   "seperated text file or entered as a string with commas "
+                   "between components of a point and spaces between points. "
+                   "points should either all be 3 componnent (x,y,z) or 6"
+                   " component (x,y,z,dx,dy,dz). If 3 component, -sdirs is "
+                   "required, if 6-component, -sdirs is ignored")
+@click.option("-sdirs", callback=np_load,
+              help="sensor directions, this can be a .npy file, a whitespace "
+                   "seperated text file or entered as a string with commas "
+                   "between components of a point and spaces between points. "
+                   "vectors should all be 3 componnent (dx,dy,dz). used with"
+                   "3-component -sensors argument, all points are run for all"
+                   "views, creating len(sensors)*len(sdirs) results. this"
+                   "is the preferred option for multiple view directions, as"
+                   "the calculations are grouped more efficiently")
+@click.option("-viewangle", default=180.)
 @click.option("-skymask", callback=clk.split_int,
               help="""mask to reduce output from full SkyData, enter as index
 rows in wea/epw file using space seperated list or python range notation:
@@ -513,30 +578,26 @@ rows in wea/epw file using space seperated list or python range notation:
     - 12:8760:24 (everyday at Noon)
 
 """)
-@click.option("-metrics", callback=clk.split_str, default="illum dgp ugp",
-              help='metrics to compute, choices: ["illum", '
-                   '"avglum", "gcr", "ugp", "dgp", "tasklum", "backlum", '
-                   '"dgp_t1", "log_gc", "dgp_t2", "ugr", "threshold", "pwsl2", '
-                   '"view_area", "backlum_true", "srcillum", "srcarea", '
-                   '"maxlum"]')
-@click.option("--hdr/--no-hdr", default=False,
-              help="make an image for every unmasked item in skydata")
-@click.option("--metric/--no-metric", default=True,
-              help="calculate metrics")
 @click.option("-res", default=800, help="image resolution")
 @click.option("--interpolate/--no-interpolate", default=False,
               help="use linear iinterpolation in image output")
-@click.option("-viewangle", default=180.0,
-              help='applies to both image and metrics, should be <= 180 or 360')
+@click.option("--namebyindex/--no-namebyindex", default=False,
+              help="if False (default), names images by: "
+                   "<prefix>_sky-<row>_pt-<x>_<y>_<z>_vd-<dx>_<dy>_<dz>.hdr "
+                   "if True, names images by: "
+                   "<prefix>_sky-<row>_pt-<pidx>_vd-<vidx>.hdr, "
+                   "where pidx, vidx refer to the order of points, and vm.")
+@click.option("--dcomp/--no-dcomp", default=True,
+              help="try loading dcomp sun points")
 @click.option("-basename", default="results",
-              help="basename for hdr outputs will write: <basename>_#####.hdr"
-                   " for each value in skymask (or range(len(skydata)) if "
-                   "skymask is None).")
+              help="prefix of namebyindex.")
+@click.option("--sunonly/--no-sunonly", default=False,
+              help="metrics for sun source only (no sky)")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
-def evaluate(ctx, sensor=None, skymask=None, metrics=None, hdr=False,
-             metric=True, res=800, interpolate=False, viewangle=180.0,
-             basename="results", **kwargs):
-    """evaluate metrics and/or make hdr for a single sensor location
+def images(ctx, sensors=None, sdirs=None, viewangle=180., skymask=None,
+           basename="results", res=800, interpolate=False, namebyindex=False,
+           sunonly=False, dcomp=True, **kwargs):
+    """render images
 
     Prequisites
     ~~~~~~~~~~~
@@ -548,8 +609,7 @@ def evaluate(ctx, sensor=None, skymask=None, metrics=None, hdr=False,
         - Invokes skydata
         - invokes area (no effects)
         - invokes suns (no effects)
-        - if hdr=True, writes: <basename>_#####.hdr
-        - if metric=True, writes: <basename>.tsv
+        - writes: output images according to --namebyindex
 
     """
     if 'scene' not in ctx.obj:
@@ -564,34 +624,161 @@ def evaluate(ctx, sensor=None, skymask=None, metrics=None, hdr=False,
     if 'skymapper' not in ctx.obj:
         clk.invoke_dependency(ctx, suns)
     skmapper = ctx.obj['skymapper']
-    if 'daylightfield' in ctx.obj:
-        df = ctx.obj['daylightfiield']
-    else:
-        df = DayLightPlaneKD(scn, f"{scn.outdir}/{pm.name}/{skmapper.name}_"
-                                  f"sunpositions.tsv",
-                             pm, f"{skmapper.name}_sun")
-    point = sensor[0:3]
-    vm = ViewMapper(sensor[3:6], viewangle)
+    sunfile = f"{scn.outdir}/{pm.name}/{skmapper.name}_sunpositions.tsv"
+    if dcomp:
+        try:
+            df = DayLightPlaneKD(scn, sunfile, pm, f"i_{skmapper.name}_sun",
+                                 includesky=not sunonly)
+        except OSError:
+            click.echo("Warning! no dcomp runs found", err=True)
+            dcomp = False
+    if not dcomp:
+        df = DayLightPlaneKD(scn, sunfile, pm, f"{skmapper.name}_sun",
+                             includesky=not sunonly)
     if skymask is not None:
         sd.mask = skymask
-    if metric:
-        result = df.evaluate(sd, point, vm, metrics=metrics)
-        data, axes, names = result.pull("sky", "metric")
-        header = "\t".join([names[1]] + list(axes[2]))
-        data = np.hstack((axes[1][:, None], data[0]))
-        fmt = ["%d"] + ["%.4f"]*len(axes[2])
-        np.savetxt(f"{basename}.tsv", data, fmt, header=header, delimiter="\t")
-    if hdr:
-        print("no implemented yet...")
+    sensors = np.atleast_2d(sensors)
+    if sensors.shape[1] == 6:
+        result = []
+        for sensor in sensors:
+            point = sensor[0:3]
+            vm = ViewMapper(sensor[3:6], viewangle)
+            result.append(df.make_images(sd, point, vm, res=res, interp=interpolate,
+                          prefix=basename, namebyindex=namebyindex))
+        result = np.concatenate(result)
+    elif sdirs is None:
+        raise ValueError("if sensors do not have directions, sdirs cannot be "
+                         "None")
+    else:
+        result = df.make_images(sd, sensors, sdirs, viewangle=viewangle,
+                                res=res, interp=interpolate, prefix=basename,
+                                namebyindex=namebyindex)
+    for d in result:
+        print(d)
+    sd.mask = None
 
 
 @main.command()
+@click.option("-sensors", callback=np_load,
+              help="sensor points, this can be a .npy file, a whitespace "
+                   "seperated text file or entered as a string with commas "
+                   "between components of a point and spaces between points. "
+                   "points should either all be 3 componnent (x,y,z) or 6"
+                   " component (x,y,z,dx,dy,dz). If 3 component, -sdirs is "
+                   "required, if 6-component, -sdirs is ignored")
+@click.option("-sdirs", callback=np_load,
+              help="sensor directions, this can be a .npy file, a whitespace "
+                   "seperated text file or entered as a string with commas "
+                   "between components of a point and spaces between points. "
+                   "vectors should all be 3 componnent (dx,dy,dz). used with"
+                   "3-component -sensors argument, all points are run for all"
+                   "views, creating len(sensors)*len(sdirs) results. this"
+                   "is the preferred option for multiple view directions, as"
+                   "the calculations are grouped more efficiently")
+@click.option("-viewangle", default=180.)
+@click.option("-skymask", callback=clk.split_int,
+              help="""mask to reduce output from full SkyData, enter as index
+rows in wea/epw file using space seperated list or python range notation:
+
+    - 370 371 372 (10AM-12PM on jan. 16th)
+    - 12:8760:24 (everyday at Noon)
+
+""")
+@click.option("-metrics", callback=clk.split_str, default="illum dgp ugp",
+              help='metrics to compute, choices: ["illum", '
+                   '"avglum", "gcr", "ugp", "dgp", "tasklum", "backlum", '
+                   '"dgp_t1", "log_gc", "dgp_t2", "ugr", "threshold", "pwsl2", '
+                   '"view_area", "backlum_true", "srcillum", "srcarea", '
+                   '"maxlum"]')
+@click.option("-basename", default="results",
+              help="LightResult object is written to basename.npz.")
+@click.option("--sunonly/--no-sunonly", default=False,
+              help="metrics for sun source only (no sky)")
+@click.option("--dcomp/--no-dcomp", default=True,
+              help="try loading dcomp sun points")
+@click.option("--npz/--no-npz", default=True,
+              help="write LightResult object to .npz, use 'raytraverse pull'"
+                   "or LightResult('basename.npz') to access results")
+@click.option("--serr/--no-serr", default=False,
+              help="include columns of sampling info/errors columns are: "
+                   "sun_pt_err, sun_pt_bin, sky_pt_err, sky_pt_bin, sun_err, "
+                   "sun_bin. 'err' is distance from queried vector to actual. "
+                   "'bin' is the unraveled idx of source vector at a 500^2 "
+                   "resolution of the mapper.")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
-def examplescript(ctx, **kwargs):
-    """print an example workflow for script based/api level access to
-     raytraverse"""
-    example = open(raytraverse.__file__.replace("__init__", "example")).read()
-    print(example)
+def evaluate(ctx, sensors=None, sdirs=None, viewangle=180., skymask=None,
+             metrics=None, basename="results", sunonly=False, dcomp=True,
+             npz=True, serr=False, **kwargs):
+    """evaluate metrics
+
+    Prequisites
+    ~~~~~~~~~~~
+        - skyrun and sunrun must be manually invoked prior to this
+
+    Effects
+    ~~~~~~~
+        - Invokes scene
+        - Invokes skydata
+        - invokes area (no effects)
+        - invokes suns (no effects)
+        - writes: <basename>.npz light result file (use "raytraverse pull" to
+          extract data views)
+
+    """
+    if 'scene' not in ctx.obj:
+        clk.invoke_dependency(ctx, scene, reload=True, overwrite=False)
+    scn = ctx.obj['scene']
+    if 'skydata' not in ctx.obj:
+        clk.invoke_dependency(ctx, skydata, reload=True)
+    sd = ctx.obj['skydata']
+    if 'planmapper' not in ctx.obj:
+        clk.invoke_dependency(ctx, area)
+    pm = ctx.obj['planmapper']
+    if 'skymapper' not in ctx.obj:
+        clk.invoke_dependency(ctx, suns)
+    skmapper = ctx.obj['skymapper']
+    sunfile = f"{scn.outdir}/{pm.name}/{skmapper.name}_sunpositions.tsv"
+    if dcomp:
+        try:
+            df = DayLightPlaneKD(scn, sunfile, pm, f"i_{skmapper.name}_sun",
+                                 includesky=not sunonly)
+        except OSError:
+            click.echo("Warning! no dcomp runs found", err=True)
+            dcomp = False
+    if not dcomp:
+        df = DayLightPlaneKD(scn, sunfile, pm, f"{skmapper.name}_sun",
+                             includesky=not sunonly)
+    if skymask is not None:
+        sd.mask = skymask
+    sensors = np.atleast_2d(sensors)
+    if sensors.shape[1] == 6:
+        result = []
+        for sensor in sensors:
+            point = sensor[0:3]
+            vm = ViewMapper(sensor[3:6], viewangle)
+            result.append(df.evaluate(sd, point, vm, metrics=metrics))
+        data = np.concatenate([r.data for r in result], axis=1)
+        ptaxis = ResultAxis(sensors, "point")
+        viewaxis = ResultAxis([0], "view")
+        result = LightResult(data, result[0].axes[0], ptaxis, viewaxis,
+                             result[0].axes[3])
+    elif sdirs is None:
+        raise ValueError("if sensors do not have directions, sdirs cannot be "
+                         "None")
+    else:
+        result = df.evaluate(sd, sensors, sdirs, viewangle=viewangle,
+                             metrics=metrics, datainfo=serr)
+    if npz:
+        result.write(f"{basename}.npz")
+    sd.mask = None
+    ctx.obj['lightresult'] = result
+
+
+@main.command()
+@clk.shared_decs(pull_decs)
+@clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
+def pull(*args, **kwargs):
+    return shared_pull(*args, **kwargs)
 
 
 @main.result_callback()
