@@ -8,7 +8,12 @@
 
 """mapper objects"""
 
-__all__ = ['MetricSet', 'PositionIndex', 'retina']
+__all__ = ['BaseMetricSet', 'MetricSet', 'MetricSet', 'FieldMetric',
+           'PositionIndex', 'retina', 'SamplingMetrics', 'MultiLumMetricSet']
 
+from raytraverse.evaluate.basemetricset import BaseMetricSet
 from raytraverse.evaluate.metricset import MetricSet
+from raytraverse.evaluate.fieldmetric import FieldMetric
 from raytraverse.evaluate.positionindex import PositionIndex
+from raytraverse.evaluate.samplingmetrics import SamplingMetrics
+from raytraverse.evaluate.multilummetricset import MultiLumMetricSet
