@@ -16,7 +16,7 @@ class Rtrace(RadianceRenderer):
     """singleton wrapper for c++ raytrraverse.crenderer.cRtrace class
 
     this class sets default arguments, helps with initialization and setting
-    cpu limits of the cRtrace instance. see raytrraverse.crenderer.cRtrace
+    cpu limits of the cRtrace instance. see raytraverse.crenderer.cRtrace
     for more details.
 
     Parameters
@@ -59,8 +59,10 @@ class Rtrace(RadianceRenderer):
     name = 'rtrace'
     #: raytraverse.crenderer.cRtrace
     engine = cRtrace
-    defaultargs = "-av 0 0 0 -aa 0 -ab 7 -ad 128 -as 0 -c 10 -as 0 -lw 1e-5"
-    directargs = "-av 0 0 0 -ab 0 -lr 0"
+    # defaultargs = "-av 0 0 0 -aa 0 -ab 7 -ad 128 -as 0 -c 10 -as 0 -lw 1e-5"
+    defaultargs = (f"-u+ -ab 16 -av 0 0 0 -aa 0 -as 0 -dc 1 -dt 0 -lr -14 -ad "
+                   f"1000 -lw 0.00004 -st 0 -ss 16")
+    directargs = "-av 0 0 0 -ab 0 -lr 0 -n 1"
     usedirect = False
 
     def __init__(self, rayargs=None, scene=None, nproc=None,
